<?php

namespace Teknisa\Libs\Controller;

use Teknisa\Libs\Util\Utilities;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Service\Menu as MenuService;


class Menu {
    /** @var  MenuService $menuService */
    private $menuService;

    /** @param MenuService $menuService */
    public function __construct(MenuService $menuService) {
        $this->menuService = $menuService;
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function buildMenu(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $concatenateGroupMenu = Utilities::getValueInFilterDataSourceByName($conditions, 'CONCATENATE_GROUP_MENU');
            $visibleMenus = Utilities::getValueInFilterDataSourceByName($conditions, 'VISIBLE_ELEMENTS');
            $urlOrigin = Utilities::getValueInFilterDataSourceByName($conditions, 'URL_ORIGIN');
            $currentModule = Utilities::getValueInFilterDataSourceByName($conditions, 'CURRENT_MODULE');
            $routesAndMenu = $this->menuService->buildMenu($concatenateGroupMenu, $visibleMenus, $urlOrigin,
                $currentModule);
            $response->addDataSet(new DataSet('menu', $routesAndMenu['MENUS']));
            $response->addDataSet(new DataSet('routesTranslate', $routesAndMenu['ROUTES_TRANSLATE']));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
} 